"use strict";
/*
* This program and the accompanying materials are made available under the terms of the *
* Eclipse Public License v2.0 which accompanies this distribution, and is available at *
* https://www.eclipse.org/legal/epl-v20.html                                      *
*                                                                                 *
* SPDX-License-Identifier: EPL-2.0                                                *
*                                                                                 *
* Copyright Contributors to the Zowe Project.                                     *
*                                                                                 *
*/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const imperative_1 = require("@zowe/imperative");
const nls = require("vscode-nls");
const path = require("path");
const url_1 = require("url");
const vscode = require("vscode");
const zowe = require("@zowe/cli");
const ZoweExplorerApiRegister_1 = require("./api/ZoweExplorerApiRegister");
const extension_1 = require("./extension"); // TODO: resolve cyclic dependency
const utils_1 = require("./utils");
const localize = nls.config({ messageFormat: nls.MessageFormat.file })(__filename);
let IConnection;
let InputBoxOptions;
class Profiles {
    constructor(log) {
        this.log = log;
        this.allProfiles = [];
        this.profilesByType = new Map();
        this.defaultProfileByType = new Map();
        this.profileManagerByType = new Map();
    }
    // Processing stops if there are no profiles detected
    static createInstance(log) {
        return __awaiter(this, void 0, void 0, function* () {
            Profiles.loader = new Profiles(log);
            yield Profiles.loader.refresh();
            return Profiles.loader;
        });
    }
    static getInstance() {
        return Profiles.loader;
    }
    loadNamedProfile(name, type) {
        for (const profile of this.allProfiles) {
            if (profile.name === name && (type ? profile.type === type : true)) {
                return profile;
            }
        }
        throw new Error(localize(0, null)
            + name + localize(1, null));
    }
    getDefaultProfile(type = "zosmf") {
        return this.defaultProfileByType.get(type);
    }
    getProfiles(type = "zosmf") {
        return this.profilesByType.get(type);
    }
    refresh() {
        return __awaiter(this, void 0, void 0, function* () {
            this.allProfiles = [];
            for (const type of ZoweExplorerApiRegister_1.ZoweExplorerApiRegister.getInstance().registeredApiTypes()) {
                const profileManager = yield this.getCliProfileManager(type);
                const profilesForType = (yield profileManager.loadAll()).filter((profile) => {
                    return profile.type === type;
                });
                if (profilesForType && profilesForType.length > 0) {
                    this.allProfiles.push(...profilesForType);
                    this.profilesByType.set(type, profilesForType);
                    this.defaultProfileByType.set(type, (yield profileManager.load({ loadDefault: true })));
                }
            }
        });
    }
    validateAndParseUrl(newUrl) {
        let url;
        const validProtocols = ["https"];
        const DEFAULT_HTTPS_PORT = 443;
        const validationResult = {
            valid: false,
            host: null,
            port: null
        };
        try {
            url = new url_1.URL(newUrl);
        }
        catch (error) {
            return validationResult;
        }
        // overkill with only one valid protocol, but we may expand profile types and protocols in the future?
        if (!validProtocols.some((validProtocol) => url.protocol.includes(validProtocol))) {
            return validationResult;
        }
        // if port is empty, set https defaults
        if (!url.port.trim()) {
            validationResult.port = DEFAULT_HTTPS_PORT;
        }
        else {
            validationResult.port = Number(url.port);
        }
        validationResult.host = url.hostname;
        validationResult.valid = true;
        return validationResult;
    }
    getUrl(urlInputBox) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve) => {
                urlInputBox.onDidHide(() => { resolve(urlInputBox.value); });
                urlInputBox.onDidAccept(() => {
                    if (this.validateAndParseUrl(urlInputBox.value).valid) {
                        resolve(urlInputBox.value);
                    }
                    else {
                        urlInputBox.validationMessage = localize(2, null);
                    }
                });
            });
        });
    }
    editSession(profileLoaded, profileName) {
        return __awaiter(this, void 0, void 0, function* () {
            const editSession = profileLoaded.profile;
            const editURL = "https://" + editSession.host + ":" + editSession.port;
            const editUser = editSession.user;
            const editPass = editSession.password;
            const updUrl = yield this.urlInfo(editURL);
            const updUser = yield this.userInfo(editUser);
            const updPass = yield this.passwordInfo(editPass);
            const updRU = yield this.ruInfo();
            const updProfile = {
                host: updUrl.host,
                port: updUrl.port,
                user: updUser,
                password: updPass,
                rejectUnauthorized: updRU,
                base64EncodedAuth: null
            };
            try {
                const updSession = yield zowe.ZosmfSession.createBasicZosmfSession(updProfile);
                updProfile.base64EncodedAuth = updSession.ISession.base64EncodedAuth;
                yield this.updateProfile({ profile: updProfile, name: profileName });
                vscode.window.showInformationMessage(localize(3, null));
                return updProfile;
            }
            catch (error) {
                yield utils_1.errorHandling(error.message);
            }
        });
    }
    createNewConnection(profileName, profileType = "zosmf") {
        return __awaiter(this, void 0, void 0, function* () {
            const newUrl = yield this.urlInfo();
            const newUser = yield this.userInfo();
            const newPass = yield this.passwordInfo();
            const newRU = yield this.ruInfo();
            for (const profile of this.allProfiles) {
                if (profile.name === profileName) {
                    vscode.window.showErrorMessage(localize(4, null));
                    return undefined;
                }
            }
            IConnection = {
                name: profileName,
                host: newUrl.host,
                port: newUrl.port,
                user: newUser,
                password: newPass,
                rejectUnauthorized: newRU
            };
            try {
                yield zowe.ZosmfSession.createBasicZosmfSession(IConnection);
                yield this.saveProfile(IConnection, IConnection.name, profileType);
                vscode.window.showInformationMessage("Profile " + profileName + " was created.");
                return profileName;
            }
            catch (error) {
                yield utils_1.errorHandling(error.message);
            }
        });
    }
    promptCredentials(sessName, rePrompt) {
        return __awaiter(this, void 0, void 0, function* () {
            let repromptUser;
            let repromptPass;
            let loadProfile;
            let loadSession;
            try {
                loadProfile = this.loadNamedProfile(sessName.trim());
                loadSession = loadProfile.profile;
            }
            catch (error) {
                yield utils_1.errorHandling(error.message);
            }
            if (rePrompt) {
                repromptUser = loadSession.user;
                repromptPass = loadSession.password;
            }
            if (!loadSession.user || rePrompt) {
                const newUser = yield this.userInfo(repromptUser);
                loadSession.user = loadProfile.profile.user = newUser;
            }
            if (!loadSession.password || rePrompt) {
                const newPass = yield this.passwordInfo(repromptPass);
                loadSession.password = loadProfile.profile.password = newPass;
            }
            try {
                const updSession = yield zowe.ZosmfSession.createBasicZosmfSession(loadSession);
                if (rePrompt) {
                    yield this.updateProfile(loadProfile);
                }
                return [updSession.ISession.user, updSession.ISession.password, updSession.ISession.base64EncodedAuth];
            }
            catch (error) {
                yield utils_1.errorHandling(error.message);
            }
        });
    }
    // ** Functions for handling Profile Information */
    urlInfo(input) {
        return __awaiter(this, void 0, void 0, function* () {
            let zosmfURL;
            const urlInputBox = vscode.window.createInputBox();
            if (input) {
                urlInputBox.value = input;
            }
            urlInputBox.ignoreFocusOut = true;
            urlInputBox.placeholder = localize(5, null);
            urlInputBox.prompt = localize(6, null);
            urlInputBox.show();
            zosmfURL = yield this.getUrl(urlInputBox);
            urlInputBox.dispose();
            if (!zosmfURL) {
                vscode.window.showInformationMessage(localize(7, null));
                return undefined;
            }
            const zosmfUrlParsed = this.validateAndParseUrl(zosmfURL);
            return zosmfUrlParsed;
        });
    }
    userInfo(input) {
        return __awaiter(this, void 0, void 0, function* () {
            let userName;
            if (input) {
                userName = input;
            }
            InputBoxOptions = {
                placeHolder: localize(8, null),
                prompt: localize(9, null),
                value: userName
            };
            userName = yield vscode.window.showInputBox(InputBoxOptions);
            if (userName === undefined) {
                vscode.window.showInformationMessage(localize(10, null));
                return;
            }
            return userName;
        });
    }
    passwordInfo(input) {
        return __awaiter(this, void 0, void 0, function* () {
            let passWord;
            if (input) {
                passWord = input;
            }
            InputBoxOptions = {
                placeHolder: localize(11, null),
                prompt: localize(12, null),
                password: true,
                value: passWord
            };
            passWord = yield vscode.window.showInputBox(InputBoxOptions);
            if (passWord === undefined) {
                vscode.window.showInformationMessage(localize(13, null));
                return;
            }
            return passWord;
        });
    }
    ruInfo() {
        return __awaiter(this, void 0, void 0, function* () {
            let rejectUnauthorize;
            const quickPickOptions = {
                placeHolder: localize(14, null),
                ignoreFocusOut: true,
                canPickMany: false
            };
            const selectRU = ["True - Reject connections with self-signed certificates",
                "False - Accept connections with self-signed certificates"];
            const ruOptions = Array.from(selectRU);
            const chosenRU = yield vscode.window.showQuickPick(ruOptions, quickPickOptions);
            if (chosenRU === ruOptions[0]) {
                rejectUnauthorize = true;
            }
            else if (chosenRU === ruOptions[1]) {
                rejectUnauthorize = false;
            }
            else {
                vscode.window.showInformationMessage(localize(15, null));
                return undefined;
            }
            return rejectUnauthorize;
        });
    }
    // ** Functions that Calls Get CLI Profile Manager  */
    updateProfile(ProfileInfo) {
        return __awaiter(this, void 0, void 0, function* () {
            for (const type of ZoweExplorerApiRegister_1.ZoweExplorerApiRegister.getInstance().registeredApiTypes()) {
                const profileManager = yield this.getCliProfileManager(type);
                this.loadedProfile = (yield profileManager.load({ name: ProfileInfo.name }));
            }
            const OrigProfileInfo = this.loadedProfile.profile;
            const NewProfileInfo = ProfileInfo.profile;
            if (OrigProfileInfo.user) {
                OrigProfileInfo.user = NewProfileInfo.user;
            }
            if (OrigProfileInfo.password) {
                OrigProfileInfo.password = NewProfileInfo.password;
            }
            OrigProfileInfo.host = NewProfileInfo.host;
            OrigProfileInfo.port = NewProfileInfo.port;
            OrigProfileInfo.rejectUnauthorized = NewProfileInfo.rejectUnauthorized;
            OrigProfileInfo.base64EncodedAuth = NewProfileInfo.base64EncodedAuth;
            const updateParms = {
                name: this.loadedProfile.name,
                merge: true,
                profile: OrigProfileInfo
            };
            try {
                (yield this.getCliProfileManager(this.loadedProfile.type)).update(updateParms);
            }
            catch (error) {
                vscode.window.showErrorMessage(error.message);
            }
        });
    }
    saveProfile(ProfileInfo, ProfileName, ProfileType) {
        return __awaiter(this, void 0, void 0, function* () {
            let zosmfProfile;
            try {
                zosmfProfile = yield (yield this.getCliProfileManager(ProfileType)).save({ profile: ProfileInfo, name: ProfileName, type: ProfileType });
            }
            catch (error) {
                vscode.window.showErrorMessage(error.message);
            }
            return zosmfProfile.profile;
        });
    }
    getCliProfileManager(type) {
        return __awaiter(this, void 0, void 0, function* () {
            let profileManager = this.profileManagerByType.get(type);
            if (!profileManager) {
                profileManager = yield new imperative_1.CliProfileManager({
                    profileRootDirectory: path.join(extension_1.getZoweDir(), "profiles"),
                    type
                });
                if (profileManager) {
                    this.profileManagerByType.set(type, profileManager);
                }
                else {
                    return undefined;
                }
            }
            return profileManager;
        });
    }
}
exports.Profiles = Profiles;

//# sourceMappingURL=Profiles.js.map
